from django.contrib.auth.decorators import login_required
from django.shortcuts import render_to_response, redirect
from django.template import RequestContext
from main.models import UserBook, Book, BookList
from models import Group, GroupUserBookList

import logging
logger = logging.getLogger(__name__)

@login_required
def createGroup(request):
    if "groupname" in request.POST:
        newGroup = Group(name = request.POST["groupname"],
                         description = request.POST["groupdescription"],
                         creator = request.user)
        newGroup.save()
        newGroup.addUserToGroup(request.user)
        newGroup.admins.add(request.user)
        
        return redirect(showGroupPage, groupId = newGroup.id)
    else:
        return render_to_response("groups/creategroup.html", RequestContext(request))

@login_required
def editGroup(request, groupId):
    group = Group.objects.get(id = groupId)
    if "groupname" in request.POST:
        if group.isUserAdmin(request.user):
            group.name = request.POST['groupname']
            group.description = request.POST['groupdescription']
            group.save()
        return redirect(showGroupPage, groupId = groupId)
    else:
        return render_to_response("groups/editgroup.html",{'group':group}, RequestContext(request))

@login_required
def joinGroup(request, groupId):
    group = Group.objects.get(id = groupId)
    group.users.add(request.user)
    
    userBookList = GroupUserBookList(owner = request.user, group=group)
    userBookList.save()
    
    return redirect(showGroupPage, groupId = groupId)

def leaveGroup(request, groupId):
    pass

def showAllGroups(request):
    groups = Group.objects.all()
    templateData = {'groups':groups}
    return render_to_response("groups/showallgroups.html", templateData, RequestContext(request))

def searchForGroup(request):
    pass

def showGroupPage(request, groupId):
    group = Group.objects.get(id = groupId)
    isUserInGroup = group.users.filter(id = request.user.id).count()
 
    templateData = {'group':group,
                    'isUserInGroup':isUserInGroup,
                    'isUserAdmin':group.isUserAdmin(request.user)}
    
    return render_to_response("groups/groupPage.html", templateData, RequestContext(request))

@login_required
def editBooksInGroup(request, groupId):
    logger.debug(groupId)
    group = Group.objects.get(id = groupId)
    bookList = GroupUserBookList.objects.get(group = group, owner = request.user)
    
    templateData  = {'bookList':bookList}
    if 'action' in request.POST and request.POST['action']=='update':
        userBookIds = []
        #params with the format "ub23" are userbook ids, so grab all of those ids into a list
        for param in request.POST:
            paramSplit = param.split('ub')
            if len(paramSplit) == 2:
                userBookIds.append(paramSplit[1])
        if len(userBookIds) > 0:
            bookList.books.clear()
            for userBookId in userBookIds:
                bookList.books.add(UserBook.objects.get(id = userBookId))
        return redirect(showGroupPage, groupId = groupId)
    else:
        return render_to_response("groups/edituserbooklist.html", templateData, RequestContext(request))
