from BeautifulSoup import BeautifulStoneSoup
import urllib, logging
from settings import OFFLINE

logger = logging.getLogger(__name__)

class GoogleBookSearch:
    def __init__(self, useLocal=False):
        if useLocal or OFFLINE:
            self.useLocal = True
        else:
            self.useLocal = False

    def getBooks(self, query, page=0):
        #results = self.getFakedBookData(query)
        if self.useLocal:
            results = self.getFakedBookData(query)
        else:
            results = self.getBookData(query, page=page)
        #results=Results()
        results.title = results.title.string
        results.resultsFound = results.find("opensearch:totalresults").string
        results.entries = []
        for entry in results.findAll('entry'):
            results.entries.append(self.parseEntry(entry))
        return results

    def getBookData(self, query, page=0):
        query = query.replace(" ", "+")
        if page > 0:
            queryUrl ="http://books.google.com/books/feeds/volumes?q=" + query + "&max-results=10&start-index=" + str(page * 10 + 1)
        else:
            queryUrl = "http://books.google.com/books/feeds/volumes?q=" + query + "&max-results=10"
        feedData = urllib.urlopen(queryUrl).read()
        return BeautifulStoneSoup(feedData)

    def getFakedBookData(self, query):
        file = 'main/testfeeds/googlebooks/harrypotter.xml'
        #\testfeeds\googlebooks\harrypotter.xml
        logger.debug("In OFFLINE mode - using %s as search source", file)
        return BeautifulStoneSoup(open(file))

    def parseEntry(self, entry):
        book = {}
        book['title'] = entry.title.string

        identifiers = entry.findAll("dc:identifier")

        book['gid'] = identifiers[0].string

        book['isbn'] = "00000"
        for identifier in identifiers:
            #@todo but there can be several isbns!
            if identifier.string.find('ISBN') >- 1:
                book['isbn'] = identifier.string.replace("ISBN:", "")

        if len(entry.findAll("dc:publisher")):
            book['publisher'] = entry.findAll("dc:publisher")[0].string

        if len(entry.findAll("dc:date")):
            book['date'] = entry.findAll("dc:date")[0].string

        if len(entry.findAll("dc:creator")):
            book['author'] = entry.findAll("dc:creator")[0].string
        else:
            book['author']="unknown"

        if len(entry.findAll(rel="http://schemas.google.com/books/2008/thumbnail")):
            book['coverUrl'] = entry.findAll(rel="http://schemas.google.com/books/2008/thumbnail")[0]['href']
        else:
            book['coverUrl'] = ""

        if len(entry.findAll("dc:description")):
            book['description'] = entry.findAll("dc:description")[0].string
        return book

    def getSingleBook(self, gid):
        if self.useLocal:
            feedData ='testfeeds/googlebooks/singlebooks/' + gid + '.xml'
            soup=BeautifulStoneSoup(open(feedData))
        else:
            queryUrl = "http://www.google.com/books/feeds/volumes/" + gid
            feedData = urllib.urlopen(queryUrl).read()
            soup = BeautifulStoneSoup(feedData)
        if soup.entry is None:
            print "book not found"
            raise BookNotFound(gid)
        return self.parseEntry(soup.entry)

class Results:
    def __init__(self, title, totalResults, entries):
        self.title = title
        self.totalResults = totalResults
        self.entries = entries

class BookNotFound(Exception):
    def __init__(self, value):
        self.value = value
    def __str__(self):
         return repr(self.value)
