import json
import urllib, logging
#from settings import OFFLIN
logger = logging.getLogger(__name__)

class GoogleBookSearch:
    def __init__(self):
        pass

    def getBooks(self, query, page=0):
        results = Results()
        resultData = self.getBookData(query, page=page)
        results.title = "results for " + query
        results.resultsFound = resultData['totalItems']
        results.entries = []
        for entry in resultData['items']:
            results.entries.append(self.parseEntry(entry))
        return results

    def getBookData(self, query, page=0):
        query = query.replace(" ", "+")
        if page > 0:
            queryUrl ="https://www.googleapis.com/books/v1/volumes?q=" + query + "&maxResults=10&startIndex=" + str(page * 10 + 1)
        else:
            queryUrl = "https://www.googleapis.com/books/v1/volumes?q=" + query + "&maxResults=10"
        return json.loads(urllib.urlopen(queryUrl).read())

    def parseEntry(self, entry):
        volumeInfo = entry['volumeInfo']
        book = {}
        book['title'] = volumeInfo['title']
        book['gid'] = entry['id']


        if 'industryIdentifiers' in volumeInfo:
            for identifier in volumeInfo['industryIdentifiers']:
                if identifier['type'] in ['ISBN_10', 'ISBN_13']:
                    book['isbn'] = identifier['identifier']
        else:
            book['isbn'] = "00000"

        if 'publisher' in volumeInfo:
            book['publisher'] = volumeInfo['publisher']

        if 'publishedDate' in volumeInfo:
            book['date'] = volumeInfo['publishedDate']

        if 'authors' in volumeInfo:
            book['author'] = volumeInfo['authors'][0]
        else:
            book['author']="unknown"

        if 'imageLinks' in volumeInfo and 'smallThumbnail' in volumeInfo['imageLinks']:
            book['coverUrl'] = volumeInfo['imageLinks']['smallThumbnail']
        else:
            book['coverUrl'] = ""

        if 'description' in volumeInfo:
            book['description'] = volumeInfo['description']

        return book

    def getSingleBook(self, gid):
        queryUrl = "https://www.googleapis.com/books/v1/volumes/" + gid
        data = json.loads(urllib.urlopen(queryUrl).read())
        
        return self.parseEntry(data)

class Results:
    def __init__(self):
        self.title = ""
        self.totalResults = ""
        self.entries = ""

class BookNotFound(Exception):
    def __init__(self, value):
        self.value = value
    def __str__(self):
         return repr(self.value)
