from django.db import models
from django.contrib.auth.models import User
from django.core.exceptions import ObjectDoesNotExist
from django.utils.encoding import smart_unicode, smart_str
from friends.models import FriendshipInvitation, Friendship
import messages
from notification.models import Notice
from booksearch.googlebooksearchv2 import GoogleBookSearch
import hashlib, httplib
from datetime import date

class BookManager(models.Manager):
    def getActualBook(self,isbn="", gid="", asin=""):
        if isbn:
            localBooks = Book.objects.filter(isbn=isbn)
        elif gid:
            localBooks = Book.objects.filter(gid=gid)
        elif asin:
            localBooks = Book.objects.filter(asin=asin)

        if localBooks:
            return localBooks[0]
        else:
            return self.lookupBook(isbn=isbn, gid=gid, asin=asin)

    def lookupBook(self,isbn="", gid="", asin=""):
        if gid:
            return self.lookupBookByGid(gid)
        elif isbn:
            pass
        else:
            assert False
            #@todo: handle no results returned
            #queryUrl = "http://books.google.com/books/feeds/volumes?q="+isbn+"&max-results=10" 
            #results = feedparser.parse(queryUrl)
            #newBook = Book(isbn=isbn,
            #               asin="",
            #               title=results.entries[0].title,
            #               author=results.entries[0].creator,
            #               coverUrl=results.entries[0].coverUrl)
            #newBook.save()
            #return newBook

    def lookupBookByGid(self,gid):
        #check if the book is already in the database, otherwise get it from google books
        if Book.objects.filter(gid=gid).count():
            return Book.objects.get(gid=gid)
        else:
            book = GoogleBookSearch().getSingleBook(gid)
            newBook = Book(gid=book['gid'],
                           isbn=book['isbn'],
                           asin="",
                           title=book['title'],
                           author=book['author'],
                           coverUrl=book['coverUrl'])
            newBook.save()
            return newBook

class Book(models.Model):
    isbn = models.CharField(max_length=20, null=True, blank=True)
    asin = models.CharField(max_length=20, null=True, blank=True)
    gid = models.CharField(max_length=30, unique=True)
    # edition=models.CharField(max_length=30, null=True, blank=True)
    title = models.CharField(max_length=200)
    author = models.CharField(max_length=200, null=True, blank=True)
    coverUrl = models.CharField(max_length=200, null=True, blank=True)

    objects = BookManager()

class UserProfile(models.Model):
    user = models.ForeignKey(User, unique=True)

    facebookId = models.CharField(max_length=100, null=True, blank=True)
    twitterUsername = models.CharField(max_length=100, null=True, blank=True)

    def getBooksOwned(self):
        return UserBook.objects.filter(owner=self.user).order_by('book__title')

    def getBooksBorrowed(self):
        return UserBook.objects.filter(currentLocation__actualUser=self.user).filter(isLent=True)

    def getBooksLent(self):
        return UserBook.objects.filter(owner=self.user).filter(isLent=True)

    def getRequestQueue(self):
        return BorrowRequest.objects.filter(userbook__owner=self.user)

    def getBooksRequested(self):
        return BorrowRequest.objects.filter(requester=self.user)

    def removeBook(self,gid="", isbn="", asin=""):
        if gid:
            userBook = UserBook.objects.filter(owner=self.user).filter(book__gid=gid)
        elif asin:
            userBook = UserBook.objects.filter(owner=self.user).filter(book__asin=asin)
        elif isbn:
            userBook = UserBook.objects.filter(owner=self.user).filter(book__isbn=isbn)

        if userBook:
            #if a book is going to be deleted, return its details as confirmation
            actualBook = userBook[0].book
            userBook.delete()
            return actualBook
        else:
            #if no book was deleted don't return anything
            return False

    def getFriends(self):
        return Friendship.objects.friends_for_user(self.user)

    def getFriendRequests(self):
        return FriendshipInvitation.objects.filter(to_user=self.user)

    def getfriendStatus(self,user):
        return Friendship.objects.are_friends(self.user,user)

    def getFriendsBooks(self):
        #friends
        pass

    def getAvatarUrl(self):
        return "http://www.gravatar.com/avatar/" + hashlib.md5(self.user.email).hexdigest() + "?d=http%3a%2f%2fwww.idlelibrary.com%2fmedia%2fimages%2fprofileimages%2fnone.png"

    def checkGravatarExists(self):
        gravatarConnection = httplib.HTTPConnection("www.gravatar.com",  port=80)
        gravatarConnection.request("GET", "avatar/" + hashlib.md5(self.user.email).hexdigest())
        gravatar = gravatarConnection.getresponse()
        if gravatar.status == "404":
            return "False"
        else:
            return "True"

    def getNotifications(self):
        return Notice.objects.notices_for(self.user)

    def getBook(self,gid):
        book = Book.objects.getActualBook(gid=gid)
        try:
            userBook = UserBook.objects.get(book=book, owner=self.user)
            return userBook
        except ObjectDoesNotExist:
            return None

    def addBook(self, gid):
        bookToAdd = Book.objects.getActualBook(gid=gid)
        if self.getBook(gid) is not None:
            return False
        else:
            newBook=UserBook(owner=self.user,
                             book=bookToAdd,
                             returndue=date.today())
            newBook.save()
            return newBook

    def getContacts(self):
        return BorrowLocation.objects.filter(relatedTo=self.user)

    def getUnreadMessages(self):
        return messages.models.inbox_count_for(self.user)

User.profile = property(lambda u: UserProfile.objects.get_or_create(user=u)[0])

class BorrowLocation(models.Model):
    actualUser = models.ForeignKey(User, null=True, blank=True, related_name="actualuser")
    relatedTo = models.ForeignKey(User ,null=True, blank=True, related_name="relatedto")
    name = models.CharField(max_length=100, null=True, blank=True)
    facebookId = models.CharField(max_length=100, null=True, blank=True)
    twitterUsername = models.CharField(max_length=100, null=True, blank=True)
    email = models.EmailField(max_length=200, null=True, blank=True)
    notes = models.CharField(max_length=1000 ,null=True, blank=True)

    def getAvatarUrl(self):
        if self.email:
            return "http://www.gravatar.com/avatar/" + hashlib.md5(self.email).hexdigest() + "?d=http%3a%2f%2fidlelibrary.com%2fmedia%2fimages%2fprofileimages%2fnone.png"

    def getBorrowed(self):
        return UserBook.objects.filter(currentLocation__id=self.id)

    def getSensibleName(self):
        if self.actualUser:
            return self.actualUser.username
        else:
            return self.name

class UserBook(models.Model):
    book = models.ForeignKey(Book)
    owner = models.ForeignKey(User)
    isLent = models.BooleanField()
    returnNotified = models.BooleanField()
    currentLocation = models.ForeignKey(BorrowLocation, related_name="location", default=False, null=True, blank=True)
    lentDate = models.DateField(null=True, blank=True)
    returndue = models.DateField(null=True, blank=True)
    #shortReview = models.CharField(max_length = 200, null=True, blank=True)

    def getRequestQueue(self):
        return BorrowRequest.objects.filter(userBook=self)

    def getBorrowHistory(self):
        pass

    def borrowedDays(self):
        timeLent = date.today() - self.lentDate
        return timeLent.days

class BookList(models.Model):
    name = models.CharField(max_length=100)
    description = models.TextField(null=True, blank=True)
    owner = models.ForeignKey(User)
    books = models.ManyToManyField(UserBook)


class BorrowRequest(models.Model):
    userbook = models.ForeignKey(UserBook)
    requester = models.ForeignKey(User)
    requestDate = models.DateField()
    status = models.CharField(max_length=20)

    AWAITING_ACCEPT = "awaitingaccept"
    AWAITING_EXCHANGE = "awaitingexchange"

    class Meta:
        unique_together = (('userbook', 'requester'),)

class BorrowHistory(models.Model):
    userbook = models.ForeignKey(UserBook)
    requester = models.ForeignKey(User)
    borrowdate = models.DateField()
    returndate = models.DateField()
