from django.test import TestCase
from main.models import UserBook,Book
from django.contrib.auth.models import User
from main.utils import DataUtils
from datetime import date
from main.booksearch.googlebooksearchv2 import GoogleBookSearch
from idlelibrary.friends.models import FriendshipInvitation, Friendship

class ProfileTest(TestCase):
    def testCreateUser(self):
        testUserName = "testuser"
        testUser= User.objects.create_user(testUserName,"a@b.com","p4ssword")
        testUser.save()
        testUser = User.objects.get(username__exact=testUserName)
        self.failUnlessEqual(testUser.username,testUserName)
    
    def testGetActualBook(self):
        testGid = "8cp-Z_G42g4C"
        data = DataUtils()
        existingBook = Book.objects.filter(gid=testGid)
        if existingBook:
            existingBook.clear()
            #log clearing the book?
            
        #get a book that doesn't exist
        testNewBook = data.getActualBook(gid=testGid)
        self.failUnlessEqual(testNewBook.gid,testGid)

    def testAddBookToUser(self):
        self.testCreateUser()
        data = DataUtils()
        testUserName="testuser"
        testGid="8cp-Z_G42g4C"
        testUser = User.objects.get(username__exact=testUserName)
        actualBook = data.getActualBook(gid=testGid)
        testUserBook = UserBook(book=actualBook,
                                owner=testUser,
                                currentLocation=testUser,
                                returndue=date.today())
        testUserBook.save()
        testUserBook = UserBook.objects.filter(book=actualBook)
        self.assertTrue(testUserBook)
        self.failUnlessEqual(testUserBook[0].owner, testUser)
    
    def testGetUsersBooks(self):
        self.testAddBookToUser()
        testUserName="testuser"
        testGid="8cp-Z_G42g4C"
        testUser = User.objects.get(username__exact=testUserName)
        testUserBooks = testUser.profile.getBooksOwned()
        self.assertTrue(testUserBooks)
        testBook=testUserBooks.filter(book__gid=testGid)
        self.failUnlessEqual(testBook[0].book.gid, str(testGid))

class SearchTest(TestCase):      
    def testBookSearch(self):
        search = GoogleBookSearch()
        books= search.getBooks("hobbit")
        print books.title
        print "found "+str(books.resultsFound)+" books"
        self.assertTrue(int(books.resultsFound) > 0)
        
    def testSingleLookUp(self):
        search=GoogleBookSearch()
        book=search.getSingleBook("8cp-Z_G42g4C")
        self.failUnlessEqual(book.title, "Pride and prejudice")
        
class FriendTest(TestCase):

    def testCreateFriendship(self):
        userA= User.objects.create_user("a","a@buzzwordoverload.co.uk","p4ssword")
        userA.save()
        userB= User.objects.create_user("b","b@buzzwordoverload.co.uk","p4ssword")
        userB.save()
        
        invite = FriendshipInvitation(from_user=userA,
                                        to_user=userB,
                                        message="friend me",
                                        status="2")
        invite.save()
        invite.accept()
        
        self.assertTrue(Friendship.objects.are_friends(userA,userB))
        
