from django.http import HttpResponse, HttpResponseRedirect
from django.template import Context,loader, RequestContext
from django.shortcuts import render_to_response
from django.contrib.auth.decorators import login_required
from django.contrib.auth.forms import UserCreationForm
from django.contrib.auth import logout
from django.contrib.sites.models import Site
from django.db import IntegrityError
from django import forms
from datetime import date, timedelta
from calendar import TextCalendar, monthcalendar
from main.models import User,UserBook,Book, BorrowRequest, BorrowLocation
from main.utils import DataUtils, safeDictGet, render_to_string
from django.core.mail import send_mail
from friends.models import FriendshipInvitation, Friendship
from main.views.users import profile, redirectToProfile
from messages.models import Message

@login_required 
def borrowBook(request):
    if 'user' in request.GET and 'gid' in request.GET:
        #add request to queue
        owner=User.objects.get(username__exact=request.GET['user'])
        book=Book.objects.getActualBook(gid=request.GET['gid'])
        userBook=UserBook.objects.get(book=book, owner=owner)
        borrowRequest=BorrowRequest(userbook=userBook, requester=request.user,
                                     requestDate=date.today(),
                                     status=BorrowRequest.AWAITING_ACCEPT)
        try:
            borrowRequest.save()
        except IntegrityError:
            return HttpResponse("book already requested. probably.")
        #notify user
        
        current_site = Site.objects.get_current()
        messageDict = {"requester":request.user,
                       "userbook": userBook,
                       'currentSite':current_site}
        
        message=render_to_string("emails/borrowrequest.txt",messageDict)
        
        message=Message(subject="borrow request",
                        body=message,
                        sender=request.user,
                        recipient=owner,
                        about_book=book,
                        )
        message.save()
        profileMessage="Request sent to "+owner.username +" to borrow "+book.title
        return redirectToProfile(request,profileMessage)
        
    else:
        return HttpResponse("invalid parameters")

@login_required 
def acceptBorrowRequestForm(request):
    if 'user' in request.GET and 'gid' in request.GET:
        book=Book.objects.getActualBook(gid=request.GET['gid'])
        #build up the calendar
        if 'date' in request.GET:
            pageDate=request.GET['date'].split(".")
            selectedDay=date(day=int(pageDate[0]),
                                month=int(pageDate[1]),
                                year=int(pageDate[2]))
        else:
            selectedDay=date(day=date.today().day,
                                month=date.today().month,
                                year=date.today().year)
        
        if selectedDay.month==12:
            nextMonth=date(day=selectedDay.day,
                                month=1,
                                year=selectedDay.year+1)
        else:
            nextMonth=date(day=selectedDay.day,
                                month=selectedDay.month+1,
                                year=selectedDay.year)
        
        if selectedDay.month==1:
            prevMonth=date(day=selectedDay.day,
                                month=12,
                                year=selectedDay.year-1)
        else:
            prevMonth=date(day=selectedDay.day,
                                month=selectedDay.month-1,
                                year=selectedDay.year)
        weeksFromToday=[]
        for week in range(1,5):
            weeksFromToday.append(date.today()+timedelta(weeks=week))
        
        calendarDays=monthcalendar(month=selectedDay.month,year=selectedDay.year)
        calendar={"calendarDays":calendarDays,
                    "selectedDay":selectedDay,
                    "nextMonth":nextMonth,
                    "prevMonth":prevMonth,
					"weeksFromToday":weeksFromToday}
        templateData={'borrowuser':User.objects.get(username__exact=request.GET['user']),
                        "gid":request.GET['gid'],
                        "book":book,
                        "calendar":calendar}
        
        return render_to_response("acceptBorrowrequestForm.html",templateData,RequestContext(request))
    else:
        return HttpResponse("invalid parameters")
   
@login_required 
def acceptBorrowRequest(request):
    if 'user' in request.GET and 'gid' in request.GET:
        print "year:"+request.POST['returndueyear']
        returnDate=date(day=int(request.POST['returndueday']),
                        month=int(request.POST['returnduemonth']),
                        year=int(request.POST['returndueyear']))
        
        #validate first
        user=User.objects.get(username__exact=request.GET['user'])
        book=Book.objects.getActualBook(gid=request.GET['gid'])
        userBook=UserBook.objects.get(book=book, owner=request.user)
        userBook.returndue=returnDate
        userBook.save()
        
        borrowRequest=BorrowRequest.objects.get(userbook=userBook, requester=user)
        borrowRequest.status=BorrowRequest.AWAITING_EXCHANGE
        borrowRequest.save()
        message = request.user.username +"acepted your request to borrow "
        message += book.title +". they want it back on "+returnDate.strftime("%d/%m/%y")
        message +="\n exchange details:\n"+request.POST['exchangedetails']
        send_mail("Idle Library borrow request accepted",message,"librarian@idlelibrary.com",['edwardmccaughan@gmail.com'])
        profileMessage="Accepted "+borrowRequest.requester.username +"'s request to borrow "+book.title
        return redirectToProfile(request,profileMessage)

@login_required     
def removeRequest(request):
    if 'borrower' in request.GET and 'gid' in request.GET:
        borrower=User.objects.get(username__exact=request.GET['borrower'])
        book=Book.objects.getActualBook(gid=request.GET['gid'])
        if 'owner' in request.GET:
            owner=User.objects.get(username__exact=request.GET['owner'])
            userBook=UserBook.objects.get(book=book, owner=owner)
            borrowRequest=BorrowRequest.objects.filter(userbook=userBook, requester=borrower)
            profileMessage="Removed your request to borrow "+book.title+" from " +owner.username
        else:
            owner=request.user
            userBook=UserBook.objects.get(book=book, owner=owner)
            borrowRequest=BorrowRequest.objects.filter(userbook=userBook, requester=borrower)
            profileMessage="Declined "+borrowRequest[0].requester.username +"'s request to borrow "+book.title
        borrowRequest[0].delete()
        return redirectToProfile(request,profileMessage)
    else:
        return HttpResponse("invalid parameters")
    
@login_required
def confirmExchange(request):
    if 'gid' in request.GET and 'action' in request.GET:    
        book=Book.objects.getActualBook(gid=request.GET['gid'])
        userBook=UserBook.objects.get(book=book, owner=request.user)
        if request.GET['action']=="borrowed":
            if 'user' in request.GET:
                borrower=User.objects.get(username__exact=request.GET['user'])
                borrowRequest=BorrowRequest.objects.filter(userbook=userBook, requester=borrower)
                borrowLocation=BorrowLocation(actualUser=borrower)
                borrowLocation.save()
                userBook.currentLocation=borrowLocation
                userBook.lentDate=date.today()
                userBook.save()
                borrowRequest.delete()
                profileMessage="Confirmed  "+borrower.username +" recieved "+book.title +" to you"
                return redirectToProfile(request,profileMessage)
            else:
                return HttpResponse("borrower not specified")
        elif request.GET['action']=="returned":
            profileMessage="Confirmed  "+userBook.currentLocation.getSensibleName() +" returned "+book.title +" to you"
            userBook.isLent=False
            userBook.save()
            
            return redirectToProfile(request,profileMessage)
        else:
            return HttpResponse("thats not written yet...")

def sendOverdueNotices(request):
	return HttpResponse("thats not written yet...")

def setBookLentTo(request):
    if 'gid' in request.GET:
        book=request.user.profile.getBook(request.GET['gid'])
        if 'borrower' in request.GET:
            if User.objects.filter(username__exact=request.GET['borrower']).count():
                borrower=User.objects.get(username__exact=request.GET['borrower'])
                setBookLocationToUser(book,borrower)
                lentTo=borrower.username
            else:
                #user doesn't exist, so display a message
                message="There doesn't seem to be a user called "+ request.GET['borrower']
                message+=". Check you have the right spelling and try again"
                return render_to_response("lenttoform.html",{"book":book,"message":message},RequestContext(request))
        elif 'existingcontact' in request.GET:
            lentTo=setBookLocationToExistingContact(book, BorrowLocation.objects.get(id=request.GET['existingcontact']))
        elif 'name' in request.GET:
            setBookLocationToNewContact(book,
                                request.user,
                                request.GET['name'],
                                request.GET['email'],
                                request.GET['facebook'],
                                request.GET['twitter'],
                                request.GET['notes'])
            lentTo=request.GET['name']
           
        else:
            return render_to_response("lenttoform.html",{"book":book},RequestContext(request))
        #unless the "else" pulls us out of the logic, display who we lent the book to
        book.lentDate=date.today()
        book.isLent=True
        book.save()
        return redirectToProfile(request,lentTo + " has "+book.book.title)

@login_required
def notifyReturn(request):
    userValue = safeDictGet(request.GET, "user")
    gidValue = safeDictGet(request.GET, "gid")
    if userValue and gidValue:
        owner = User.objects.get(username__exact=userValue)
        userBook = owner.profile.getBook(gidValue)
        if userBook is not None:
            emailTitle = request.user.username + " returned " + userBook.book.title
            current_site = Site.objects.get_current()
            message = render_to_string("emails/notifyreturn.txt", {"user":request.user,"userBook": userBook,'currentSite':current_site})
            send_mail(emailTitle,message,"librarian@idlelibrary.com",[owner.email])
        
            userBook.returnNotified = True
            userBook.save()
            return redirectToProfile(request, owner.username + " was notified you've returned "+userBook.book.title)
        else:
            pass


def setBookLocationToUser(book,borrower): 
    borrowLocation=BorrowLocation(actualUser=borrower)
    borrowLocation.save()
    book.currentLocation=borrowLocation
    book.save()
    #check if theres a request for this book, then remove it
    if BorrowRequest.objects.filter(userbook=book, requester=borrower).count():
        borrowRequest=BorrowRequest.objects.get(userbook=book, requester=borrower)
        borrowRequest.delete()

def setBookLocationToExistingContact(book,contact):
    book.currentLocation=contact
    book.save()
    return contact.name

def setBookLocationToNewContact(book,relateduser,name,email,facebook,twitter,notes):
    borrowLocation=BorrowLocation(relatedTo=relateduser,
                                  name=name,
                                  email=email,
                                  facebookId=facebook,
                                  twitterUsername=twitter,
                                  notes=notes)
    borrowLocation.save()
    book.currentLocation=borrowLocation
    book.save()


    