from django.http import HttpResponse, HttpResponseRedirect
from django.template import Context,loader, RequestContext
from django.shortcuts import render_to_response
from django.contrib.auth.decorators import login_required
from django.contrib.auth.forms import UserCreationForm
from django.contrib.auth import logout
from django.contrib.sites.models import Site
from django.core.mail import send_mail
from django import forms
from datetime import date
from main.models import User,UserBook,Book, BorrowRequest
from main.utils import DataUtils
from friends.models import FriendshipInvitation, Friendship
from main.views.users import redirectToProfile,showUserProfile
from messages.models import Message

from django.template.loader import render_to_string


@login_required
def sendFriendRequest(request):
    if 'user' in request.GET:
        newFriend = User.objects.get(username__exact=request.GET['user'])
        
        #check if they've sent you a friend request first
        existingInvite=FriendshipInvitation.objects.filter(from_user=newFriend,
                                                    to_user=request.user)
        if existingInvite.count():
            return confirmFriendRequest(request)
        elif Friendship.objects.are_friends(newFriend,request.user):
            return redirectToProfile(request,"You're already friends")
        else:
            invite=FriendshipInvitation(from_user=request.user,
                                to_user=newFriend,
                                message="add me as a friend",
                                status="2")
            invite.save()
            sendFriendRequestMessage(request.user,newFriend)
            return redirectToProfile(request,"friend request sent")
    else:
        return HttpResponse("no friend specified")

def sendFriendRequestMessage(user,newFriend):
    current_site = Site.objects.get_current()
    message=render_to_string('emails/confirmfriendrequest.txt',{'friend':user,'currentSite':current_site})
    message=Message(subject="Idle library Friend Request",
                    body=message,
                    sender=user,
                    recipient=newFriend
                    )
    message.save()
    


@login_required     
def confirmFriendRequest(request):
    if 'user' in request.GET:
        to_user = User.objects.get(username__exact=request.GET['user'])
        invite=FriendshipInvitation.objects.filter(from_user=to_user,
                                                    to_user=request.user)
        if invite.count():
            invite[0].accept()
            invite.delete()
            return showUserProfile(request,request.GET['user'], message="You are now friends with "+request.GET['user'])
        else:
            return redirectToProfile(request,"You are already friends with "+request.GET['user'])

@login_required        
def declineFriendRequest(request):
    if 'user' in request.GET:
        to_user = User.objects.get(username__exact=request.GET['user'])
        invite=FriendshipInvitation.objects.filter(from_user=to_user,
                                                    to_user=request.user)
        if invite.count():
            invite[0].decline()
            invite.delete()
            return redirectToProfile(request,"friend request declined")
        else:
            return redirectToProfile(request,"er, they never sent you a friend request")
            
@login_required            
def removeFriend(request):
    if 'user' in request.GET:
        friend = User.objects.get(username__exact=request.GET['user'])
        Friendship.objects.remove(request.user,friend)
        return redirectToProfile(request,"removed "+friend.username+" from your list of friends")