from main.utils import safeDictGet, render_to_string
from django.shortcuts import render_to_response
from django.contrib.sites.models import Site
from main.models import User
from settings import ADMINS
from django.http import HttpResponse
from django.core.mail import send_mail
import logging

log = logging.getLogger(__name__)


def emailSummary(request,username=""):
    if username:
        #if given a username, send the message
        user=User.objects.get(username__exact=username)
        message = sendEmailSummary(user,send=safeDictGet(request.GET, "send"))
        return HttpResponse(message)
    else:
        #otherwise send *everyone's* reminders
        message="Sending all messages:\n"
        for user in User.objects.all():
            if hasContentToEmail(user):
                message+=user.username + ":\n"
                message+=sendEmailSummary(user,send=safeDictGet(request.GET, "send"))
        if safeDictGet(request.GET, "ccadmin")=="true":
            send_mail("Idle Library books summary - admin digest",message,"librarian@idlelibrary.com",ADMINS[0])
        return HttpResponse(message)

def sendEmailSummary(user, send=False):
    current_site = Site.objects.get_current()
    message = render_to_string("emails/bookssummary.txt",{'user':user,'currentSite':current_site})
    if send:
        log.debug("sending summary email to "+user.email)
        try:
            send_mail("Idle Library books summary",message,"librarian@idlelibrary.com",[user.email])
        except SMTPRecipientsRefused:
            log.error("sending email to user:[" + user.username + "] at email:[" + user.email + "] failed due to SMTPRecipientsRefused")
    return message

def hasContentToEmail(user):
    if (user.profile.getBooksBorrowed() or user.profile.getBooksLent() or user.profile.getRequestQueue()):
        return True
    else:
        return False